@@ -550,6 +550,7 @@ static void
 ephy_embed_shell_setup_process_model (EphyEmbedShell *shell)
 {
   EphyPrefsProcessModel process_model;
+  guint max_processes;
 
   if (ephy_embed_shell_get_mode (shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     process_model = EPHY_PREFS_PROCESS_MODEL_SHARED_SECONDARY_PROCESS;
@@ -558,14 +559,15 @@ ephy_embed_shell_setup_process_model (EphyEmbedShell *shell)
 
   switch (process_model) {
   case EPHY_PREFS_PROCESS_MODEL_SHARED_SECONDARY_PROCESS:
-    webkit_web_context_set_process_model (shell->priv->web_context,
-                                          WEBKIT_PROCESS_MODEL_SHARED_SECONDARY_PROCESS);
+    max_processes = 1;
     break;
   case EPHY_PREFS_PROCESS_MODEL_ONE_SECONDARY_PROCESS_PER_WEB_VIEW:
-    webkit_web_context_set_process_model (shell->priv->web_context,
-                                          WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
+    max_processes = g_settings_get_uint (EPHY_SETTINGS_MAIN, EPHY_PREFS_MAX_PROCESSES);
     break;
   }
+
+  webkit_web_context_set_process_model (shell->priv->web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
+  webkit_web_context_set_web_process_count_limit (shell->priv->web_context, max_processes);
 }
 
 static void
