@@ -66,9 +66,6 @@ EphyFindToolbar *ephy_find_toolbar_new		 (WebKitWebView *web_view);
 
 const char	*ephy_find_toolbar_get_text	 (EphyFindToolbar *toolbar);
 
-void		 ephy_find_toolbar_set_web_view	 (EphyFindToolbar *toolbar,
-						  WebKitWebView *web_view);
-
 void		 ephy_find_toolbar_find_next	 (EphyFindToolbar *toolbar);
 
 void		 ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar);
@@ -81,6 +78,8 @@ void		 ephy_find_toolbar_close	 (EphyFindToolbar *toolbar);
 
 void		 ephy_find_toolbar_request_close (EphyFindToolbar *toolbar);
 
+void		 ephy_find_toolbar_toggle_state	 (EphyFindToolbar *toolbar);
+
 G_END_DECLS
 
 #endif /* EPHY_FIND_TOOLBAR_H */
