@@ -78,6 +78,7 @@ struct _EphyWebViewPrivate {
   guint ever_committed : 1;
 
   char *address;
+  char *display_address;
   char *typed_address;
   char *loading_message;
   char *link_message;
@@ -489,8 +490,8 @@ ephy_web_view_create_form_auth_save_confirmation_info_bar (EphyWebView *web_view
 
   LOG ("Going to show infobar about %s", webkit_web_view_get_uri (WEBKIT_WEB_VIEW (web_view)));
 
-  info_bar = gtk_info_bar_new_with_buttons (_("_Save"), GTK_RESPONSE_YES,
-                                            _("_Don’t Save"), GTK_RESPONSE_CLOSE,
+  info_bar = gtk_info_bar_new_with_buttons (_("_Don’t Save"), GTK_RESPONSE_CLOSE,
+                                            _("_Save"), GTK_RESPONSE_YES,
                                             NULL);
 
   action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
@@ -791,6 +792,7 @@ ephy_web_view_finalize (GObject *object)
   ephy_web_view_popups_manager_reset (EPHY_WEB_VIEW (object));
 
   g_free (priv->address);
+  g_free (priv->display_address);
   g_free (priv->typed_address);
   g_free (priv->link_message);
   g_free (priv->loading_message);
@@ -828,7 +830,7 @@ title_changed_cb (WebKitWebView *web_view,
   if (!title && uri)
     title = title_from_address = ephy_embed_utils_get_title_from_address (uri);
 
-  if (uri && title && !ephy_web_view_is_history_frozen (webview))
+  if (uri && title && *title && !ephy_web_view_is_history_frozen (webview))
     ephy_history_service_set_url_title (history, uri, title, NULL, NULL, NULL);
 
   g_free (title_from_address);
@@ -854,6 +856,9 @@ ephy_web_view_set_address (EphyWebView *view,
   g_free (priv->address);
   priv->address = g_strdup (address);
 
+  g_free (priv->display_address);
+  priv->display_address = g_uri_unescape_string (priv->address, NULL);
+
   is_blank = address == NULL ||
              strcmp (address, "about:blank") == 0;
   _ephy_web_view_set_is_blank (view, is_blank);
@@ -961,7 +966,7 @@ ephy_web_view_class_init (EphyWebViewClass *klass)
 /**
  * EphyWebView:address:
  *
- * View's current address.
+ * View's current address. This is a percent-encoded URI.
  **/
   g_object_class_install_property (gobject_class,
                                    PROP_ADDRESS,
@@ -1522,9 +1527,11 @@ ephy_web_view_set_loading_message (EphyWebView *view,
 
   g_clear_pointer (&priv->loading_message, g_free);
   if (address) {
+    char *decoded_address;
     char *title;
 
-    title = ephy_embed_utils_get_title_from_address (address);
+    decoded_address = g_uri_unescape_string (address, NULL);
+    title = ephy_embed_utils_get_title_from_address (decoded_address);
 
     if (title != NULL && title[0] != '\0') {
       /* translators: %s here is the address of the web page */
@@ -1533,6 +1540,7 @@ ephy_web_view_set_loading_message (EphyWebView *view,
       priv->loading_message = g_strdup (_("Loading…"));
     }
 
+    g_free (decoded_address);
     g_free (title);
   }
 
@@ -2347,7 +2355,9 @@ ephy_web_view_is_overview (EphyWebView *view)
  * ephy_web_view_get_address:
  * @view: an #EphyWebView
  *
- * Returns the address of the currently loaded page.
+ * Returns the address of the currently-loaded page, percent-encoded.
+ * This URI should not be displayed to the user; to do that, use
+ * ephy_web_view_get_display_address().
  *
  * Return value: @view's address. Will never be %NULL.
  **/
@@ -2358,6 +2368,24 @@ ephy_web_view_get_address (EphyWebView *view)
   return priv->address ? priv->address : "about:blank";
 }
 
+/**
+ * ephy_web_view_get_display_address:
+ * @view: an #EphyWebView
+ *
+ * Returns the display address of the currently-loaded page. This is a
+ * decoded URI suitable for display to the user. To get a URI suitable
+ * for sending to a server, e.g. for storage in the bookmarks or history
+ * database, use ephy_web_view_get_address().
+ *
+ * Return value: @view's address. Will never be %NULL.
+ */
+const char *
+ephy_web_view_get_display_address (EphyWebView *view)
+{
+  EphyWebViewPrivate *priv = view->priv;
+  return priv->display_address ? priv->display_address : "about:blank";
+}
+
 /**
  * ephy_web_view_is_loading:
  * @view: an #EphyWebView
@@ -2486,16 +2514,17 @@ ephy_web_view_get_link_message (EphyWebView *view)
 /**
  * ephy_web_view_set_link_message:
  * @view: an #EphyWebView
- * @link_message: new value for link-message in @view
+ * @address: new value for link-message in @view
  *
  * Sets the value of link-message property which tells the URL of the hovered
  * link.
  **/
 void
 ephy_web_view_set_link_message (EphyWebView *view,
-                                const char *link_message)
+                                const char *address)
 {
   EphyWebViewPrivate *priv;
+  char *decoded_address;
 
   g_return_if_fail (EPHY_IS_WEB_VIEW (view));
 
@@ -2503,7 +2532,10 @@ ephy_web_view_set_link_message (EphyWebView *view,
 
   g_free (priv->link_message);
 
-  priv->link_message = ephy_embed_utils_link_message_parse (link_message);
+  decoded_address = g_uri_unescape_string (address, NULL);
+  priv->link_message = ephy_embed_utils_link_message_parse (decoded_address);
+
+  g_free (decoded_address);
 
   g_object_notify (G_OBJECT (view), "status-message");
   g_object_notify (G_OBJECT (view), "link-message");
