@@ -1478,6 +1478,16 @@ provide_keyword_uri (EphyNode *node, GValue *value, gpointer data)
 	g_free (uri);
 }
 
+static void
+unescape_bookmark_uri (EphyNode *node,
+		       GValue *value,
+		       gpointer user_data)
+{
+	const char *url = g_value_get_string (value);
+	g_value_take_string (value, g_uri_unescape_string (url, NULL));
+}
+
+
 static void
 ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 {
@@ -1629,11 +1639,12 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	gtk_tree_view_column_set_min_width (editor->priv->title_col, 300);
 	gtk_tree_view_column_set_resizable (editor->priv->title_col, TRUE);
 
-	url_col_id = ephy_node_view_add_column
+	url_col_id = ephy_node_view_add_column_full
 				  (EPHY_NODE_VIEW (bm_view), _("Address"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_LOCATION,
 				   EPHY_NODE_VIEW_SORTABLE |
 				   EPHY_NODE_VIEW_ELLIPSIZED,
+				   unescape_bookmark_uri, NULL,
 				   NULL, &(editor->priv->address_col));
 	gtk_tree_view_column_set_min_width (editor->priv->address_col, 300);
 	gtk_tree_view_column_set_resizable (editor->priv->address_col, TRUE);
